@@ -32,6 +32,12 @@ fi
 PYTHON="$1"
 ICDIFF="icdiff"
 
+if [ ! -z "$INSTALLED" ]; then
+  INVOCATION="$ICDIFF"
+else
+  INVOCATION="$PYTHON $ICDIFF"
+fi
+
 function fail() {
   echo "FAIL"
   exit 1
@@ -47,11 +53,7 @@ function check_gold() {
 
   echo "    check_gold $gold matches $@"
   local tmp=/tmp/icdiff.output
-  if [ ! -z "$INSTALLED" ]; then
-    "$ICDIFF" "$@" &> $tmp
-  else
-    "$PYTHON" "$ICDIFF" "$@" &> $tmp
-  fi
+  $INVOCATION "$@" &> $tmp
 
   if $REGOLD; then
     if [ -e $gold ] && diff $tmp $gold > /dev/null; then
@@ -79,6 +81,30 @@ function check_gold() {
   fi
 }
 
+FIRST_TIME_CHECK_GIT_DIFF=true
+function check_git_diff() {
+  local gitdiff=tests/$1
+  shift
+
+  echo "    check_gitdiff $gitdiff matches git icdiff $@"
+  # Check when using icdiff in git
+  if $FIRST_TIME_CHECK_GIT_DIFF; then
+    FIRST_TIME_CHECK_GIT_DIFF=false
+    # Set default args when first time check git diff
+    yes | git difftool --extcmd icdiff > /dev/null
+    git config --global icdiff.options '--cols=80'
+    export PATH=$PATH:"$(pwd)"
+  fi
+  local tmp=/tmp/git-icdiff.output
+  git icdiff $1 $2 &> $tmp
+  if ! diff $tmp $gitdiff; then
+    echo "Got: ($tmp)"
+    cat $tmp
+    echo "Expected: ($gitdiff)"
+    fail
+  fi
+}
+
 check_gold gold-recursive.txt       --recursive tests/{a,b} --cols=80
 check_gold gold-exclude.txt         --exclude-lines '^#|  pad' tests/input-4-cr.txt tests/input-4-partial-cr.txt --cols=80
 check_gold gold-dir.txt             tests/{a,b} --cols=80
@@ -117,13 +143,17 @@ check_gold gold-12-subcolors.txt    tests/input-{1,2}.txt --cols=80 --color-map=
 check_gold gold-subcolors-bad-color tests/input-{1,2}.txt --cols=80 --color-map='change:mageta,description:cyan_bold'
 check_gold gold-subcolors-bad-cat tests/input-{1,2}.txt --cols=80 --color-map='chnge:magenta,description:cyan_bold'
 check_gold gold-subcolors-bad-fmt tests/input-{1,2}.txt --cols=80 --color-map='change:magenta:gold,description:cyan_bold'
+check_gold gold-bad-encoding.txt tests/input-{1,2}.txt --encoding=nonexistend_encoding
+check_git_diff gitdiff-only-newlines.txt 4e86205629~1 4e86205629
 
-
-if [ ! -z "$INSTALLED" ]; then
-  VERSION=$(icdiff --version | awk '{print $NF}')
-else
-  VERSION=$(./icdiff --version | awk '{print $NF}')
+# Testing pipe behavior doesn't fit well with the check_gold system
+$INVOCATION tests/input-{4,5}.txt 2>/tmp/icdiff-pipe-error-output | head -n 1
+if [ -s /tmp/icdiff-pipe-error-output ]; then
+  echo 'emitting errors on early pipe closure'
+  fail
 fi
+
+VERSION=$($INVOCATION --version | awk '{print $NF}')
 if [ "$VERSION" != $(head -n 1 ChangeLog) ]; then
   echo "Version mismatch between ChangeLog and icdiff source."
   fail
